var Debug = (function () {
    function Debug() { }
    Debug.debuggingEnabled = true;
    Debug.log = function log(str) {
        if(this.debuggingEnabled == true) {
            console.log(str);
        }
    }
    Debug.warn = function warn(str) {
        if(this.debuggingEnabled == true) {
            throw str;
        }
    }
    return Debug;
})();
var KeyInput = (function () {
    function KeyInput() {
        var _this = this;
        this._pressed = null;
        this._pressed = new Array();
        window.addEventListener('keyup', function (event) {
            return _this.onKeyUp(event);
        }, false);
        window.addEventListener('keydown', function (event) {
            return _this.onKeyDown(event);
        }, false);
    }
    KeyInput.LEFT = 37;
    KeyInput.UP = 38;
    KeyInput.RIGHT = 39;
    KeyInput.DOWN = 40;
    KeyInput.CTRL = 17;
    KeyInput.SHIFT = 16;
    KeyInput.TAB = 9;
    KeyInput.W = 87;
    KeyInput.S = 83;
    KeyInput.A = 65;
    KeyInput.D = 68;
    KeyInput.prototype.keyIsDown = function (keycode) {
        return this._pressed[keycode];
    };
    KeyInput.prototype.onKeyDown = function (keycode) {
        this._pressed[event.keyCode] = true;
    };
    KeyInput.prototype.onKeyUp = function (keycode) {
        this._pressed[event.keyCode] = false;
    };
    return KeyInput;
})();
var MathUtils = (function () {
    function MathUtils() { }
    MathUtils.degreesToRadians = function degreesToRadians(degrees) {
        return degrees * Math.PI / 180;
    }
    MathUtils.distanceBetween = function distanceBetween(point1, point2) {
        return Math.sqrt((point1.x - point2.x) * (point1.x - point2.x) + (point1.y - point2.y) * (point1.y - point2.y));
    }
    return MathUtils;
})();
var Asset = (function () {
    function Asset() {
        this.type = null;
        this.url = null;
        this.tag = null;
        this.atlasData = null;
        this.atlasDataRequest = null;
        this.content = null;
    }
    return Asset;
})();
var AtlasFrameData = (function () {
    function AtlasFrameData() {
        this.frameX = 0;
        this.frameY = 0;
        this.frameWidth = 0;
        this.frameHeight = 0;
        this.filename = null;
    }
    return AtlasFrameData;
})();
var Point = (function () {
    function Point(xval, yval) {
        if (typeof xval === "undefined") { xval = 0; }
        if (typeof yval === "undefined") { yval = 0; }
        this.xval = xval;
        this.yval = yval;
        this.x = 0;
        this.y = 0;
        this.x = xval;
        this.y = yval;
    }
    Point.prototype.distanceToPoint = function (point) {
        return Math.sqrt((this.x - point.x) * (this.x - point.x) + (this.y - point.y) * (this.y - point.y));
    };
    return Point;
})();
var Rect = (function () {
    function Rect(x, y, w, h) {
        this._x = 0;
        this._y = 0;
        this._width = 0;
        this._height = 0;
        this._left = 0;
        this._right = 0;
        this._top = 0;
        this._bottom = 0;
        this._x = x;
        this._y = y;
        this._width = w;
        this._height = h;
        this._top = y;
        this._bottom = y + this._height;
        this._left = x;
        this._right = x + this._width;
    }
    Object.defineProperty(Rect.prototype, "x", {
        get: function () {
            return this._x;
        },
        set: function (val) {
            this._x = val;
            this._left = val;
            this._right = val + this._width;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rect.prototype, "y", {
        get: function () {
            return this._y;
        },
        set: function (val) {
            this._y = val;
            this._top = val;
            this._bottom = val + this._height;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rect.prototype, "width", {
        get: function () {
            return this._width;
        },
        set: function (val) {
            this._width = val;
            this._right = this._x + val;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rect.prototype, "height", {
        get: function () {
            return this._height;
        },
        set: function (val) {
            this._height = val;
            this._bottom = this._y + val;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rect.prototype, "left", {
        get: function () {
            return this._left;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rect.prototype, "right", {
        get: function () {
            return this._right;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rect.prototype, "top", {
        get: function () {
            return this._top;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rect.prototype, "bottom", {
        get: function () {
            return this._bottom;
        },
        enumerable: true,
        configurable: true
    });
    Rect.prototype.containsPoint = function (point) {
        return false;
    };
    Rect.prototype.containsRect = function (rect) {
        return false;
    };
    Rect.prototype.intersectsRect = function (rect) {
        return !(this.left > rect.right || this.right < rect.left || this.top > rect.bottom || this.bottom < rect.top);
    };
    return Rect;
})();
var ScreenData = (function () {
    function ScreenData(canvas, width, height) {
        this._width = 0;
        this._height = 0;
        this._canvas = null;
        this._width = width;
        this._height = height;
        this._canvas = canvas;
        this._canvas.height = height;
        this._canvas.width = width;
    }
    Object.defineProperty(ScreenData.prototype, "width", {
        get: function () {
            return this._width;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ScreenData.prototype, "height", {
        get: function () {
            return this._height;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ScreenData.prototype, "canvas", {
        get: function () {
            return this._canvas;
        },
        enumerable: true,
        configurable: true
    });
    ScreenData.prototype.setBackgroundImage = function (url, repeat) {
        if (typeof repeat === "undefined") { repeat = false; }
        switch(repeat) {
            case true: {
                this._canvas.style.background = "url(" + url + ")";
                this._canvas.style.backgroundRepeat = "repeat";
                break;

            }
            case false: {
                this._canvas.style.background = "url(" + url + ")";
                this._canvas.style.backgroundRepeat = "no-repeat";
                break;

            }
        }
    };
    return ScreenData;
})();
var Component = (function () {
    function Component() { }
    return Component;
})();
var Entity = (function () {
    function Entity(tag) {
        this._tag = null;
        this._components = null;
        this._componentsCount = 0;
        this._components = new Array();
        this._tag = tag;
        Debug.log("Entity* created: " + this._tag);
    }
    Entity.prototype.components = function () {
        return this._components;
    };
    Entity.prototype.componentsCount = function () {
        return this._componentsCount;
    };
    Entity.prototype.tag = function () {
        return this._tag;
    };
    Entity.prototype.componentAddWithTag = function (component, tag) {
        if(this._components[tag] == undefined) {
            this._components[tag] = component;
            this._componentsCount++;
            Debug.log("Entity* Component: " + tag + " added to Entity: " + this._tag);
        } else {
            Debug.warn("Entity* tag: " + tag + " already used for Entity: " + this._tag);
        }
    };
    Entity.prototype.componentGetWithTag = function (tag) {
        if(this._components[tag] != undefined) {
            return this._components[tag];
        } else {
            Debug.warn("Entity* tag: " + tag + " already used for Entity: " + this._tag);
        }
    };
    Entity.prototype.componentRemoveWithTag = function (tag) {
        if(this._components[tag] != undefined) {
            this._componentsCount--;
            this._components[tag] = undefined;
            Debug.log("Entity* Component: " + tag + " removed from Entity: " + this._tag);
        } else {
            Debug.warn("Entity* tag: " + tag + " already used for Entity: " + this._tag);
        }
    };
    Entity.prototype.onAddedToGame = function () {
        Debug.log("Entity* " + this._tag + " added to game");
    };
    Entity.prototype.onRemoveFromGame = function () {
        Debug.log("Entity* " + this._tag + " removed from game");
    };
    return Entity;
})();
var System = (function () {
    function System() {
        this.tag = null;
        this.nodelist = null;
        this.nodelist = new SystemNodeList();
    }
    System.prototype.start = function () {
    };
    System.prototype.update = function (dt, t) {
    };
    System.prototype.end = function () {
    };
    return System;
})();
var SystemNode = (function () {
    function SystemNode(entity) {
        this.entity = entity;
    }
    return SystemNode;
})();
var SystemNodeList = (function () {
    function SystemNodeList() {
        this.head = null;
        this.tail = null;
    }
    SystemNodeList.prototype.add = function (node) {
        if(this.head == undefined) {
            this.head = this.tail = node;
            node.next = node.previous = null;
        } else {
            this.tail.next = node;
            node.previous = this.tail;
            node.next = null;
            this.tail = node;
        }
    };
    SystemNodeList.prototype.remove = function (node) {
        if(this.head == node) {
            this.head = this.head.next;
        }
        if(this.tail == node) {
            this.tail = this.tail.previous;
        }
        if(node.previous) {
            node.previous.next = node.next;
        }
        if(node.next) {
            node.next.previous = node.previous;
        }
    };
    SystemNodeList.prototype.removeAll = function () {
        while(this.head) {
            var node = this.head;
            this.head = node.next;
            node.previous = null;
            node.next = null;
        }
        this.tail = null;
    };
    return SystemNodeList;
})();
var Engine = (function () {
    function Engine(canvas, width, height) {
        this.managerEntity = null;
        this.managerSystem = null;
        this.managerAsset = null;
        this.managerNode = null;
        this._initialTime = 0;
        this._deltaTime = 0;
        this._timeElapsed = 0;
        this._timeElapsedTotal = 0;
        this._timeLast = 0;
        this._timeScale = 1;
        this._gameIsPaused = false;
        this.managerSystem = new ManagerSystem();
        this.managerEntity = new ManagerEntity();
        this.managerAsset = new ManagerAsset();
        this.managerNode = new ManagerSystemNode();
        this.managerSystem.addWithTag(new SystemMovement(), "movement", ManagerSystem.PRIORITY_UPDATE);
        this.managerSystem.addWithTag(new SystemAnimation(), "animation", ManagerSystem.PRIORITY_UPDATE);
        this.managerSystem.addWithTag(new SystemCanvasRenderer(new ScreenData(canvas, width, height)), "renderer", ManagerSystem.PRIORITY_RENDER);
        Debug.log("Engine* init complete");
    }
    Engine.prototype.setCoreRunning = function (val) {
        if(val == true) {
            this._initialTime = new Date().getTime();
            this.main(this);
        } else {
        }
    };
    Engine.prototype.main = function (scope) {
        window.requestAnimationFrame(function () {
            return scope.main(scope);
        });
        var newTime = new Date().getTime() - scope._initialTime;
        var frameTime = newTime - scope._timeLast;
        scope._deltaTime = frameTime / 1000;
        scope._timeLast = newTime;
        scope._timeElapsedTotal += frameTime;
        if(scope._gameIsPaused) {
            return;
        } else {
            scope._timeElapsed += frameTime;
            scope.managerSystem.update(scope._deltaTime, scope._timeElapsed);
        }
    };
    return Engine;
})();
var __extends = this.__extends || function (d, b) {
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var ComponentPosition = (function (_super) {
    __extends(ComponentPosition, _super);
    function ComponentPosition() {
        _super.apply(this, arguments);

        this.x = 0;
        this.y = 0;
        this.rotation = 0;
        this.origin = new Point(0, 0);
    }
    return ComponentPosition;
})(Component);
var ComponentAnimation = (function (_super) {
    __extends(ComponentAnimation, _super);
    function ComponentAnimation() {
        _super.apply(this, arguments);

        this._frames = null;
        this.currentFrame = null;
        this._currentAnimationTag = null;
        this.frameskip = 0;
        this.frameskipCounter = 0;
        this.frameCounter = 0;
        this._frameAnimations = [];
    }
    Object.defineProperty(ComponentAnimation.prototype, "frames", {
        get: function () {
            return this._frames;
        },
        set: function (data) {
            this._frames = data;
            this.addWithTag("default", [
                "default"
            ]);
            this.currentFrame = this._frames["default"];
            this._currentAnimationTag = "default";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ComponentAnimation.prototype, "currentAnimationTag", {
        get: function () {
            return this._currentAnimationTag;
        },
        set: function (tag) {
            if(this._frameAnimations[tag] != undefined) {
                this._currentAnimationTag = tag;
            } else {
                Debug.warn("ComponentAnimation* no animation defined: " + tag);
            }
        },
        enumerable: true,
        configurable: true
    });
    ComponentAnimation.prototype.addWithTag = function (tag, frames) {
        if(this._frameAnimations[tag] == undefined) {
            this._frameAnimations[tag] = frames;
        } else {
            Debug.warn("ComponentAnimation* animation already defined: " + tag);
        }
    };
    ComponentAnimation.prototype.getWithTag = function (tag) {
        if(this._frameAnimations[tag] != undefined) {
            return this._frameAnimations[tag];
        } else {
            Debug.warn("ComponentAnimation* animation not defined: " + tag);
        }
    };
    return ComponentAnimation;
})(Component);
var ComponentVelocity = (function (_super) {
    __extends(ComponentVelocity, _super);
    function ComponentVelocity() {
        _super.apply(this, arguments);

        this.x = 0;
        this.y = 0;
        this.damping = 0.98;
    }
    return ComponentVelocity;
})(Component);
var ComponentDisplay = (function (_super) {
    __extends(ComponentDisplay, _super);
    function ComponentDisplay() {
        _super.apply(this, arguments);

        this._view = null;
        this._width = 0;
        this._height = 0;
        this._scaleX = 1;
        this._scaleY = 1;
        this._origWidth = 0;
        this._origHeight = 0;
    }
    Object.defineProperty(ComponentDisplay.prototype, "view", {
        get: function () {
            return this._view;
        },
        set: function (view) {
            this._view = view;
            this._height = view.height;
            this._width = view.width;
            this._origWidth = view.width;
            this._origHeight = view.height;
            this._scaleX = 1;
            this._scaleY = 1;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ComponentDisplay.prototype, "width", {
        get: function () {
            return this._width;
        },
        set: function (value) {
            this._width = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ComponentDisplay.prototype, "height", {
        get: function () {
            return this._height;
        },
        set: function (value) {
            this._height = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ComponentDisplay.prototype, "scaleX", {
        get: function () {
            return this._scaleX;
        },
        set: function (value) {
            this._scaleX = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ComponentDisplay.prototype, "scaleY", {
        get: function () {
            return this._scaleY;
        },
        set: function (value) {
            this._scaleY = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ComponentDisplay.prototype, "scale", {
        get: function () {
            return new Point(this._scaleX, this._scaleY);
        },
        set: function (value) {
            this._scaleX = value.x;
            this._scaleY = value.y;
        },
        enumerable: true,
        configurable: true
    });
    return ComponentDisplay;
})(Component);
var ManagerEntity = (function () {
    function ManagerEntity() {
        this._entities = null;
        this._entitiesCount = 0;
        if(ManagerEntity.instance == null) {
            ManagerEntity.instance = this;
            this._entities = new Array();
        }
        Debug.log("ManagerEntity* init");
    }
    ManagerEntity.instance = null;
    ManagerEntity.prototype.entityCount = function () {
        return this._entitiesCount;
    };
    ManagerEntity.prototype.getWithTag = function (tag) {
        if(this._entities[tag] != undefined) {
            return this._entities[tag];
        } else {
            Debug.warn("ManagerEntity* tag not used");
        }
    };
    ManagerEntity.prototype.addWithTag = function (entity, tag) {
        if(this._entities[tag] == undefined) {
            this._entities[tag] = entity;
            this._entitiesCount++;
            entity.onAddedToGame();
        } else {
            Debug.warn("ManagerEntity* tag already used for Entity");
        }
    };
    ManagerEntity.prototype.removeWithTag = function (tag) {
        if(this._entities[tag] != undefined) {
            this._entities[tag].onRemoveFromGame();
            this._entities[tag] = undefined;
            this._entitiesCount--;
        } else {
            Debug.warn("ManagerEntity* tag not found: " + tag);
        }
    };
    return ManagerEntity;
})();
var ManagerAsset = (function () {
    function ManagerAsset() {
        this._assets = null;
        this._queue = null;
        this.preloading = false;
        this._returnFunction = null;
        if(ManagerAsset.instance == null) {
            ManagerAsset.instance = this;
            this._assets = new Array();
            this._queue = new Array();
        }
        Debug.log("ManagerAsset* init");
    }
    ManagerAsset.instance = null;
    ManagerAsset.TYPE_IMAGE = "image_type";
    ManagerAsset.TYPE_AUDIO = "audio_type";
    ManagerAsset.TYPE_ATLAS = "atlas_type";
    ManagerAsset.prototype.addWithTag = function (assetURL, type, tag) {
        var _this = this;
        if(this._assets[tag] == undefined) {
            var asset = new Asset();
            switch(type) {
                case ManagerAsset.TYPE_IMAGE: {
                    asset.type = type;
                    asset.url = assetURL;
                    asset.tag = tag;
                    asset.content = new Image();
                    asset.content.onload = function () {
                        return _this.preloadNextItem(_this);
                    };
                    asset.content.onerror = function () {
                        return Debug.log("error");
                    };
                    break;

                }
                case ManagerAsset.TYPE_ATLAS: {
                    var jsonURL = assetURL.replace(".png", "").replace(".jpg", "").replace(".jpeg", "").replace(".bmp", "") + '.json';
                    var request = new XMLHttpRequest();
                    request.overrideMimeType("application/json");
                    request.onreadystatechange = function (responseText) {
                        if(this.readyState == 4 && this.status == 200) {
                            var jsonData = JSON.parse(this.responseText);
                            var frameData = new Array();
                            for(var i = 0; i < jsonData.frames.length; i++) {
                                var frame = new AtlasFrameData();
                                frame.frameX = jsonData.frames[i].frame.x;
                                frame.frameY = jsonData.frames[i].frame.y;
                                frame.frameWidth = jsonData.frames[i].frame.w;
                                frame.frameHeight = jsonData.frames[i].frame.h;
                                frame.filename = jsonData.frames[i].filename;
                                frameData[jsonData.frames[i].filename] = frame;
                                if(i == 0) {
                                    frameData["default"] = frame;
                                }
                            }
                            this.asset.atlasData = frameData;
                            this.asset.content.src = this.asset.url;
                            this.asset.atlasDataRequest = null;
                        }
                    };
                    request.jsonURL = jsonURL;
                    request.asset = asset;
                    asset.type = type;
                    asset.url = assetURL;
                    asset.tag = tag;
                    asset.atlasDataRequest = request;
                    asset.content = new Image();
                    asset.content.onload = function () {
                        return _this.preloadNextItem(_this);
                    };
                    asset.content.onerror = function () {
                        return Debug.log("error");
                    };
                    break;

                }
            }
            this._queue.push(asset);
        } else {
            return;
        }
    };
    ManagerAsset.prototype.queueLoad = function (notify) {
        if(this._queue.length > 0 && this.preloading == false) {
            Debug.log("ManagerAsset* preload start");
            this._returnFunction = notify;
            this.preloading = true;
            switch(this._queue[0].type) {
                case ManagerAsset.TYPE_IMAGE: {
                    this._queue[0].content.src = this._queue[0].url;
                    break;

                }
                case ManagerAsset.TYPE_ATLAS: {
                    this._queue[0].atlasDataRequest.open("GET", this._queue[0].atlasDataRequest.jsonURL, true);
                    this._queue[0].atlasDataRequest.send();
                    break;

                }
            }
        } else {
            if(this.preloading == true) {
                Debug.warn("ManagerAsset* currently in preload queue");
            }
        }
    };
    ManagerAsset.prototype.getWithTag = function (tag) {
        if(this._assets[tag] != undefined) {
            return this._assets[tag];
        } else {
            Debug.warn("ManagerAsset* tag not used: " + tag);
        }
    };
    ManagerAsset.prototype.preloadNextItem = function (scope) {
        var asset = this._queue[0];
        scope._queue = scope._queue.splice(1, scope._queue.length);
        scope._assets[asset.tag] = asset;
        Debug.log("ManagerAsset* preloaded: " + asset.url);
        if(asset.type == ManagerAsset.TYPE_IMAGE) {
        }
        if(scope._queue.length > 0) {
            switch(scope._queue[0].type) {
                case ManagerAsset.TYPE_IMAGE: {
                    scope._queue[0].content.src = scope._queue[0].url;
                    break;

                }
                case ManagerAsset.TYPE_ATLAS: {
                    scope._queue[0].atlasDataRequest.open("GET", this._queue[0].atlasDataRequest.jsonURL, true);
                    scope._queue[0].atlasDataRequest.send();
                    break;

                }
            }
        } else {
            scope.preloading = false;
            Debug.log("ManagerAsset* preload complete");
            this._returnFunction();
        }
    };
    return ManagerAsset;
})();
var ManagerSystemNode = (function () {
    function ManagerSystemNode() {
        this._nodeList = null;
        if(ManagerSystemNode.instance == null) {
            ManagerSystemNode.instance = this;
            this._nodeList = new Array();
        }
        Debug.log("ManagerSystemNode* init");
    }
    ManagerSystemNode.instance = null;
    ManagerSystemNode.prototype.addToList = function (node, type) {
        if(this._nodeList[type.name] == undefined) {
            this._nodeList[type.name] = new SystemNodeList();
            Debug.log("ManagerSystemNode* created node list: " + type.name);
        }
        this._nodeList[type.name].add(node);
    };
    ManagerSystemNode.prototype.initList = function (type) {
        if(this._nodeList[type.name] == undefined) {
            this._nodeList[type.name] = new SystemNodeList();
        }
    };
    ManagerSystemNode.prototype.getList = function (type) {
        if(this._nodeList[type.name] != undefined) {
            return this._nodeList[type.name];
        } else {
            this.initList(type);
            return this._nodeList[type.name];
        }
    };
    return ManagerSystemNode;
})();
var ManagerSystem = (function () {
    function ManagerSystem() {
        this._systems_input = null;
        this._systems_update = null;
        this._systems_collision = null;
        this._systems_render = null;
        if(ManagerSystem.instance == null) {
            ManagerSystem.instance = this;
            this._systems_input = new Array();
            this._systems_update = new Array();
            this._systems_collision = new Array();
            this._systems_render = new Array();
        }
        Debug.log("ManagerSystem* init");
    }
    ManagerSystem.instance = null;
    ManagerSystem.PRIORITY_INPUT = "priority_input";
    ManagerSystem.PRIORITY_UPDATE = "priority_update";
    ManagerSystem.PRIORITY_COLLISION = "priority_collision";
    ManagerSystem.PRIORITY_RENDER = "priority_render";
    ManagerSystem.prototype.addWithTag = function (system, tag, priority) {
        var prioritisedSystem;
        switch(priority) {
            case ManagerSystem.PRIORITY_INPUT: {
                prioritisedSystem = this._systems_input;
                break;

            }
            case ManagerSystem.PRIORITY_UPDATE: {
                prioritisedSystem = this._systems_update;
                break;

            }
            case ManagerSystem.PRIORITY_COLLISION: {
                prioritisedSystem = this._systems_collision;
                break;

            }
            case ManagerSystem.PRIORITY_RENDER: {
                prioritisedSystem = this._systems_render;
                break;

            }
        }
        if(system[tag] == undefined) {
            system.tag = tag;
            prioritisedSystem[tag] = system;
            system.start();
            Debug.log("ManagerSystem* added system: " + tag);
        } else {
            Debug.warn("ManagerSystem* system tag already used: " + tag);
        }
    };
    ManagerSystem.prototype.getWithTag = function (tag) {
        if(this._systems_input[tag] != undefined) {
            return this._systems_input[tag];
        } else {
            if(this._systems_update[tag] != undefined) {
                return this._systems_update[tag];
            } else {
                if(this._systems_collision[tag] != undefined) {
                    return this._systems_collision[tag];
                } else {
                    if(this._systems_render[tag] != undefined) {
                        return this._systems_render[tag];
                    } else {
                        Debug.warn("ManagerSystem* tag not found: " + tag);
                    }
                }
            }
        }
    };
    ManagerSystem.prototype.update = function (dt, t) {
        for(var system in this._systems_input) {
            if(this._systems_input[system] != undefined) {
                this._systems_input[system].update(dt, t);
            }
        }
        for(var system in this._systems_update) {
            if(this._systems_update[system] != undefined) {
                this._systems_update[system].update(dt, t);
            }
        }
        for(var system in this._systems_collision) {
            if(this._systems_collision[system] != undefined) {
                this._systems_collision[system].update(dt, t);
            }
        }
        for(var system in this._systems_render) {
            if(this._systems_render[system] != undefined) {
                this._systems_render[system].update(dt, t);
            }
        }
    };
    ManagerSystem.prototype.removeWithTag = function (tag) {
        if(this._systems_input[tag] != undefined) {
            this._systems_input[tag].end();
            this._systems_input[tag] = undefined;
            Debug.log("ManagerSystem* removed system: " + tag);
        } else {
            if(this._systems_update[tag] != undefined) {
                this._systems_update[tag].end();
                this._systems_update[tag] = undefined;
                Debug.log("ManagerSystem* removed system: " + tag);
            } else {
                if(this._systems_collision[tag] != undefined) {
                    this._systems_collision[tag].end();
                    this._systems_collision[tag] = undefined;
                    Debug.log("ManagerSystem* removed system: " + tag);
                } else {
                    if(this._systems_render[tag] != undefined) {
                        this._systems_render[tag].end();
                        this._systems_render[tag] = undefined;
                        Debug.log("ManagerSystem* removed system: " + tag);
                    } else {
                        Debug.warn("ManagerSystem* tag not found: " + tag);
                    }
                }
            }
        }
    };
    return ManagerSystem;
})();
var SystemCanvasRenderer = (function (_super) {
    __extends(SystemCanvasRenderer, _super);
    function SystemCanvasRenderer(screenData) {
        _super.call(this);
        this._screenData = null;
        this._ctx = null;
        this._screenData = screenData;
        this._ctx = screenData.canvas.getContext('2d');
        this.nodelist = ManagerSystemNode.instance.getList(NodeRenderable);
        Debug.log("SystemCanvasRenderer* init complete");
    }
    SystemCanvasRenderer.prototype.start = function () {
        Debug.log("SystemCanvasRenderer* start");
    };
    SystemCanvasRenderer.prototype.update = function (dt, t) {
        this._ctx.clearRect(0, 0, this._screenData.width, this._screenData.height);
        for(var node = this.nodelist.head; node; node = node.next) {
            this._ctx.save();
            if(node.animation == null) {
                this._ctx.translate(node.position.x, node.position.y);
                this._ctx.rotate(node.position.rotation * Math.PI / 180);
                this._ctx.drawImage(node.display.view, 0, 0, node.display.view.width, node.display.view.height, -node.position.origin.x, -node.position.origin.y, node.display.width, node.display.height);
            } else {
                if(node.animation.currentFrame != undefined) {
                    this._ctx.translate(node.position.x, node.position.y);
                    this._ctx.rotate(node.position.rotation * Math.PI / 180);
                    this._ctx.drawImage(node.display.view, node.animation.currentFrame.frameX, node.animation.currentFrame.frameY, node.animation.currentFrame.frameWidth, node.animation.currentFrame.frameHeight, -node.position.origin.x, -node.position.origin.y, node.display.width, node.display.height);
                }
            }
            this._ctx.restore();
        }
    };
    SystemCanvasRenderer.prototype.end = function () {
        Debug.log("SystemCanvasRenderer* end");
    };
    return SystemCanvasRenderer;
})(System);
var SystemMovement = (function (_super) {
    __extends(SystemMovement, _super);
    function SystemMovement() {
        _super.call(this);
        this.nodelist = ManagerSystemNode.instance.getList(NodeMovement);
        Debug.log("SystemMovement* init complete");
    }
    SystemMovement.prototype.start = function () {
        Debug.log("SystemMovement* start");
    };
    SystemMovement.prototype.update = function (dt, t) {
        for(var node = this.nodelist.head; node; node = node.next) {
            node.position.x += node.velocity.x * dt;
            node.position.y += node.velocity.y * dt;
            node.velocity.x *= node.velocity.damping;
            node.velocity.y *= node.velocity.damping;
        }
    };
    SystemMovement.prototype.end = function () {
        Debug.log("SystemCanvasRenderer* end");
    };
    return SystemMovement;
})(System);
var SystemAnimation = (function (_super) {
    __extends(SystemAnimation, _super);
    function SystemAnimation() {
        _super.call(this);
        this._ctx = null;
        this.nodelist = ManagerSystemNode.instance.getList(NodeAnimation);
        Debug.log("SystemAnimation* init complete");
    }
    SystemAnimation.prototype.start = function () {
        Debug.log("SystemAnimation* start");
    };
    SystemAnimation.prototype.update = function (dt, t) {
        for(var node = this.nodelist.head; node; node = node.next) {
            var currentAnimTag = node.animation.currentAnimationTag;
            if(node.animation.frameCounter < node.animation.getWithTag(currentAnimTag).length) {
                if(node.animation.frameskipCounter < node.animation.frameskip) {
                    node.animation.frameskipCounter++;
                } else {
                    var filename = node.animation.getWithTag(currentAnimTag)[node.animation.frameCounter];
                    node.animation.currentFrame = node.animation.frames[filename];
                    node.animation.frameCounter++;
                    node.animation.frameskipCounter = 0;
                    node.display.width = node.animation.currentFrame.frameWidth;
                    node.display.height = node.animation.currentFrame.frameHeight;
                }
            } else {
                node.animation.frameCounter = 0;
            }
        }
    };
    SystemAnimation.prototype.end = function () {
        Debug.log("SystemAnimation* end");
    };
    return SystemAnimation;
})(System);
var SystemLogic = (function (_super) {
    __extends(SystemLogic, _super);
    function SystemLogic() {
        _super.call(this);
        this.nodelist = ManagerSystemNode.instance.getList(NodeLogic);
        Debug.log("SystemLogic* init complete");
    }
    SystemLogic.prototype.start = function () {
        Debug.log("SystemLogic* start");
    };
    SystemLogic.prototype.update = function (dt, t) {
        for(var node = this.nodelist.head; node; node = node.next) {
        }
    };
    SystemLogic.prototype.end = function () {
        Debug.log("SystemLogic* end");
    };
    return SystemLogic;
})(System);
var NodeMovement = (function (_super) {
    __extends(NodeMovement, _super);
    function NodeMovement() {
        _super.apply(this, arguments);

        this.velocity = null;
        this.position = null;
    }
    return NodeMovement;
})(SystemNode);
var NodeRenderable = (function (_super) {
    __extends(NodeRenderable, _super);
    function NodeRenderable() {
        _super.apply(this, arguments);

        this.display = null;
        this.position = null;
        this.animation = null;
    }
    return NodeRenderable;
})(SystemNode);
var NodeAnimation = (function (_super) {
    __extends(NodeAnimation, _super);
    function NodeAnimation() {
        _super.apply(this, arguments);

        this.animation = null;
        this.display = null;
    }
    return NodeAnimation;
})(SystemNode);
var NodeLogic = (function (_super) {
    __extends(NodeLogic, _super);
    function NodeLogic() {
        _super.apply(this, arguments);

        this.position = null;
        this.velocity = null;
        this.display = null;
        this.animation = null;
    }
    return NodeLogic;
})(SystemNode);
var Sprite = (function (_super) {
    __extends(Sprite, _super);
    function Sprite(tag, img, atlas) {
        _super.call(this, tag);
        this.position = null;
        this.display = null;
        this.animation = null;
        this.velocity = null;
        this.position = new ComponentPosition();
        this.display = new ComponentDisplay();
        this.velocity = new ComponentVelocity();
        var renderNode = new NodeRenderable(this);
        renderNode.display = this.display;
        renderNode.position = this.position;
        var movementNode = new NodeMovement(this);
        movementNode.velocity = this.velocity;
        movementNode.position = this.position;
        ManagerSystemNode.instance.addToList(renderNode, NodeRenderable);
        ManagerSystemNode.instance.addToList(movementNode, NodeMovement);
        this.componentAddWithTag(this.position, "position");
        this.componentAddWithTag(this.display, "display");
        this.componentAddWithTag(this.velocity, "velocity");
        this.display.view = ManagerAsset.instance.getWithTag(img).content;
        if(atlas) {
            this.animation = new ComponentAnimation();
            renderNode.animation = this.animation;
            var animationNode = new NodeAnimation(this);
            animationNode.animation = this.animation;
            animationNode.display = this.display;
            this.animation.frames = ManagerAsset.instance.getWithTag(img).atlasData;
            this.display.width = this.animation.currentFrame.frameWidth;
            this.display.height = this.animation.currentFrame.frameHeight;
            ManagerSystemNode.instance.addToList(animationNode, NodeAnimation);
            this.componentAddWithTag(this.animation, "animation");
        }
        ManagerEntity.instance.addWithTag(this, this.tag());
    }
    return Sprite;
})(Entity);
