/**
 * Game
 */
class Game extends Engine
{
	// properties
	private _keyInput:KeyInput 						= null;
	private _screenData:ScreenData 					= null;

	// game related
	private _tileSize:number						= 64;
	private _map:any = 	   [[[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9]],
							[[1,9],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[1,9]],
							[[1,9],[0,4],[1,3],[0,4],[1,3],[0,4],[1,3],[0,4],[0,4],[0,4],[1,3],[0,4],[0,4],[0,4],[0,4],[1,9]],
							[[1,9],[0,4],[0,4],[0,4],[0,4],[0,4],[1,3],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[1,9]],
							[[1,9],[0,4],[0,4],[0,4],[1,3],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[1,9]],
							[[1,9],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[1,9]],
							[[1,9],[0,4],[0,4],[0,4],[1,3],[0,4],[1,3],[0,4],[1,3],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[1,9]],
							[[1,9],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[1,9]],
							[[1,9],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[1,3],[0,4],[0,4],[0,4],[0,4],[1,9]],
							[[1,9],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[0,4],[1,9]],
							[[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9],[1,9]]];


	// constructor
	constructor(canvas, width, height)
	{
		super(canvas, width, height);

		// Debug
		//Debug.debuggingEnabled = false;

		// create the screen bounds and keyboard
		this._screenData 		= new ScreenData(canvas, width, height);
		this._keyInput 			= new KeyInput();

		// preload assets
		//this.managerAsset.addWithTag("./bin/assets/image/bg.png", 			ManagerAsset.TYPE_IMAGE, "bg");
		this.managerAsset.addWithTag("./bin/assets/image/player.png", 	ManagerAsset.TYPE_ATLAS, "playerAtlas");
		this.managerAsset.addWithTag("./bin/assets/image/tiles.png", 	ManagerAsset.TYPE_ATLAS, "tilesAtlas");
		this.managerAsset.queueLoad(() => this.postPreload());
	}



	public postPreload():void
	{
		// systems init
		this.managerSystem.addWithTag(new SystemKeyboardInput(this._keyInput, this._screenData, this._map), "main", ManagerSystem.PRIORITY_INPUT);
		//this.managerSystem.addWithTag(new SystemSimpleCollison(this._screenData), "main", ManagerSystem.PRIORITY_INPUT);

		// bg
		//this._screenData.setBackgroundImage("./bin/assets/image/bg.png", false);

		// create tiles
		var cnt:number = 0;
		for(var i:number = 0; i < this._map.length; i++)
		{
			for(var j:number = 0; j < this._map[0].length; j++)
			{
				var tile:Sprite = this.createTileType(cnt, this._map[i][j][1]);
				tile.position.x = j * this._tileSize;
				tile.position.y = (i * this._tileSize) + this._tileSize;

				cnt++;
			}
		}

		// player
		var player:Sprite = this.createPlayer();

		// start the core
		this.setCoreRunning(true);
	}


	public createTileType(id:number, type:number):Sprite
	{
		var spr:Sprite = new Sprite(id.toString(), "tilesAtlas", true);
		spr.animation.addWithTag("idle", ['tile' + type + '.png']);
		spr.animation.currentAnimationTag = "idle";

		return spr;
	}

	public createPlayer():Sprite
	{
		// player
		var player:Sprite 			= new Sprite("player", "playerAtlas", true);
		player.position.origin.x 	= 32;
		player.position.origin.y 	= 32;
		player.position.x			= (64 * 14) - 32;
		player.position.y			= (64 * 3) - 32;
		//ship.velocity.damping	= 0.99;

		// anim define
		player.animation.addWithTag("walk_right", 		["player_facing_r.png"]);
		player.animation.addWithTag("walk_rightdown", 	["player_facing_dr.png"]);
		player.animation.addWithTag("walk_down", 		["player_facing_d.png"]);
		player.animation.addWithTag("walk_leftdown", 	["player_facing_dl.png"]);
		player.animation.addWithTag("walk_left",		["player_facing_l.png"]);
		player.animation.addWithTag("walk_leftup",		["player_facing_ul.png"]);
		player.animation.addWithTag("walk_up",			["player_facing_u.png"]);
		player.animation.addWithTag("walk_rightup",		["player_facing_ur.png"]);
		player.animation.addWithTag("dead",				["player_dead.png"]);

		player.animation.currentAnimationTag = "walk_right";

		// register the ship with a node and a system that controls it
		var playerNode:PlayerNode 	= new PlayerNode(player);
		playerNode.position			= player.position;
		playerNode.display			= player.display;
		playerNode.velocity			= player.velocity;
		playerNode.animation		= player.animation;

		ManagerSystemNode.instance.addToList(playerNode, PlayerNode);

		return player;
	}
}