/**
 * Engine
 */
class Engine
{
	// managers
	public managerEntity:ManagerEntity 			= null;
	public managerSystem:ManagerSystem 			= null;
	public managerAsset:ManagerAsset 			= null;
	public managerNode:ManagerSystemNode 		= null;

	// timer related
	private _initialTime:number 				= 0;
	private _deltaTime:number 					= 0;
	private _timeElapsed:number 				= 0;
	private _timeElapsedTotal:number 			= 0;
	private _timeLast:number 					= 0;
	private _timeScale:number 					= 1.0;

	// game related
	private _gameIsPaused:bool 					= false;


	// general init
	constructor(canvas, width, height)
	{
		// init the managers
		this.managerSystem = new ManagerSystem();
		this.managerEntity = new ManagerEntity();
		this.managerAsset  = new ManagerAsset();
		this.managerNode   = new ManagerSystemNode();

		// init core systems
		this.managerSystem.addWithTag(new SystemMovement(), "movement", ManagerSystem.PRIORITY_UPDATE);
		this.managerSystem.addWithTag(new SystemAnimation(), "animation", ManagerSystem.PRIORITY_UPDATE);
		this.managerSystem.addWithTag(new SystemCanvasRenderer(new ScreenData(canvas, width, height)), "renderer", ManagerSystem.PRIORITY_RENDER);

		// output
		Debug.log("Engine* init complete");
	}



	public setCoreRunning(val:bool):void
	{
		if(val == true)
		{
			// init a timer for steps
			this._initialTime = new Date().getTime();
			this.main(this);
		} else
		{
			// TODO cancel timer...
		}
	}


	// internal engine loop
	public main(scope):void
	{
		window.requestAnimationFrame(() => scope.main(scope));

		// time related
		var newTime:number = new Date().getTime() - scope._initialTime;
		var frameTime:number = newTime - scope._timeLast;

		scope._deltaTime = frameTime / 1000;
		scope._timeLast = newTime;
		scope._timeElapsedTotal += frameTime;

		if(scope._gameIsPaused)
		{
			return;
		} else
		{
			scope._timeElapsed += frameTime;

			// update systems
			scope.managerSystem.update(scope._deltaTime, scope._timeElapsed);
		}
	}
}