/**
 * Entity
 */
class Entity
{
	// properties
	private _tag:string						= null;
	private _components:Component[]			= null;
	private _componentsCount:number			= 0;

	// getters / setters
	public components():Component[]			{ return this._components; }
	public componentsCount():number			{ return this._componentsCount; }
	public tag():string						{ return this._tag; }



	/**
	 * constructor
	 */
	constructor(tag:string)
	{
		this._components 	= new Component[];
		this._tag			= tag;

		Debug.log("Entity* created: " + this._tag);
	}


	// add a component to the entity
	public componentAddWithTag(component:Component, tag:string):void
	{
		if(this._components[tag] == undefined)
		{
			this._components[tag] = component;
			this._componentsCount++;

			Debug.log("Entity* Component: " + tag + " added to Entity: " + this._tag);
		} else
		{
			Debug.warn("Entity* tag: " + tag + " already used for Entity: " + this._tag);
		}
	}


	public componentGetWithTag(tag:string)
	{
		if(this._components[tag] != undefined)
		{
			return this._components[tag];
		} else
		{
			Debug.warn("Entity* tag: " + tag + " already used for Entity: " + this._tag);
		}
	}


	public componentRemoveWithTag(tag:string):void
	{
		if(this._components[tag] != undefined)
		{
			this._componentsCount--;
			this._components[tag] = undefined

			Debug.log("Entity* Component: " + tag + " removed from Entity: " + this._tag);
		} else
		{
			Debug.warn("Entity* tag: " + tag + " already used for Entity: " + this._tag);
		}
	}

	public onAddedToGame():void
	{
		Debug.log("Entity* " + this._tag + " added to game");
	}


	public onRemoveFromGame():void
	{
		Debug.log("Entity* " + this._tag + " removed from game");
	}
}