class ComponentAnimation extends Component
{
	// frames returned from JSON
	private _frames:AtlasFrameData[]				= null;

	// frame properties
	public currentFrame:AtlasFrameData				= null;
	private _currentAnimationTag:string				= null;

	// animation properties
	public frameskip:number							= 0;
	public frameskipCounter:number					= 0;
	public frameCounter:number						= 0;

	// animations added as: ["tag", [frames.filename, frames.filename, frames.filename]]
	private _frameAnimations:any					= [];



	// getters
	public get frames():AtlasFrameData[]			{ return this._frames; }
	public get currentAnimationTag():string			{ return this._currentAnimationTag; }



	/**
	 * setter for frames
	 *
	 * @property data - AtlasFrameData[]
	 */
	public set frames(data:AtlasFrameData[])
	{
		this._frames = data;

		// set default anim / one time thing
		this.addWithTag("default", ["default"]);
		this.currentFrame 			= this._frames["default"];
		this._currentAnimationTag 	= "default";
	}



	/**
	 * setter for currentAnimationTag
	 *
	 * @property tag - string
	 */
	public set currentAnimationTag(tag:string)
	{
		if(this._frameAnimations[tag] != undefined)
		{
			this._currentAnimationTag = tag;
		} else
		{
			Debug.warn("ComponentAnimation* no animation defined: " + tag);
		}
	}



	/**
	 * add an animation with tag
	 *
	 * @property tag 		- string animation tag
	 * @property frames 	- string[] array of frame ids defined in the JSON file
	 */
	public addWithTag(tag:string, frames:string[]):void
	{
		if(this._frameAnimations[tag] == undefined)
		{
			this._frameAnimations[tag] = frames;
		} else
		{
			Debug.warn("ComponentAnimation* animation already defined: " + tag);
		}
	}



	/**
	 * get an animation by tag
	 *
	 * @property tag 		- string animation tag
	 * @property frames 	- string[] array of frame ids defined in the JSON file
	 */
	public getWithTag(tag:string):string[]
	{
		if(this._frameAnimations[tag] != undefined)
		{
			return this._frameAnimations[tag];
		} else
		{
			Debug.warn("ComponentAnimation* animation not defined: " + tag);
		}
	}
}
