/**
 *	ManagerAsset is responsible for preloading / managing asset reuse
 */
 class ManagerAsset
 {
	// members
	public static instance:ManagerAsset = null;

	private _assets:Asset[]				= null;
	private _queue:Asset[]				= null;
	public preloading:bool				= false;
	private _returnFunction:Function 	= null;

	// static types for checking
	public static TYPE_IMAGE:string 	= "image_type";
	public static TYPE_AUDIO:string 	= "audio_type";
	public static TYPE_ATLAS:string		= "atlas_type";



	// constructor
	constructor()
	{
		// setup instance
		if(ManagerAsset.instance == null)
		{
			ManagerAsset.instance = this;

			this._assets = new Asset[];
			this._queue = new Asset[];
		}

		Debug.log("ManagerAsset* init");
	}



	public addWithTag(assetURL:string, type:string, tag:string):void
	{
		if(this._assets[tag] == undefined)
		{
			var asset:Asset = new Asset();

			switch(type)
			{
				case ManagerAsset.TYPE_IMAGE:
					asset.type 				= type;
					asset.url 				= assetURL;
					asset.tag 				= tag;
					asset.content 			= new Image();
					asset.content.onload 	= () => this.preloadNextItem(this);
					asset.content.onerror 	= () => Debug.log("error");
					break;

				case ManagerAsset.TYPE_ATLAS:
					// JSON loading / cast as any to get the compiler to stfu
					var jsonURL:string 			= assetURL.replace(".png", "").replace(".jpg", "").replace(".jpeg", "").replace(".bmp", "") + '.json'; // get path minus extension
					//var request = (window.XMLHttpRequest) ? new XMLHttpRequest() : new ActiveXObject("Microsoft.XMLHTTP"); // IE ?!
					var request:any 			= new XMLHttpRequest();
					request.overrideMimeType("application/json");
					request.onreadystatechange 	= function (responseText)
					{
						// if load success
						if(this.readyState == 4 && this.status == 200)
						{
							// parse the JSON data
							var jsonData = JSON.parse(this.responseText)
							var frameData:AtlasFrameData[] = new AtlasFrameData[];
							for(var i:number = 0; i < jsonData.frames.length; i++)
							{
								var frame:AtlasFrameData = new AtlasFrameData();
								frame.frameX 		= jsonData.frames[i].frame.x;
								frame.frameY 		= jsonData.frames[i].frame.y;
								frame.frameWidth 	= jsonData.frames[i].frame.w;
								frame.frameHeight 	= jsonData.frames[i].frame.h;
								frame.filename 		= jsonData.frames[i].filename;

								// push to frameData
								frameData[jsonData.frames[i].filename] = frame;

								// store a default frame
								if(i == 0) frameData["default"] = frame;
							}



							// load atlas image (phase 2)
							this.asset.atlasData 		= frameData;
							this.asset.content.src 		= this.asset.url;
							this.asset.atlasDataRequest = null;
						}

						// TODO erroring?
					};

					// store json data in atlasInfo param
					request.jsonURL 		= jsonURL;
					request.asset 			= asset;

					// asset setup
					asset.type 				= type;
					asset.url 				= assetURL;
					asset.tag 				= tag;
					asset.atlasDataRequest 	= request;

					// load image data
					asset.content 			= new Image();
					asset.content.onload 	= () => this.preloadNextItem(this);
					asset.content.onerror 	= () => Debug.log("error");

					break;

				// TODO Audio
			}

			// push to queue
			this._queue.push(asset);

		} else
		{
			return;
		}
	}



	public queueLoad(notify:() => any):void
	{
		// start at 0
		if(this._queue.length > 0 && this.preloading == false)
		{
			Debug.log("ManagerAsset* preload start");

			// store for invoke after complete
			this._returnFunction = notify;

			// enforce block for repeat preload inits
			this.preloading = true;

			// start at 0
			switch(this._queue[0].type)
			{
				case ManagerAsset.TYPE_IMAGE:
					this._queue[0].content.src = this._queue[0].url;
					break;

				case ManagerAsset.TYPE_ATLAS:
					this._queue[0].atlasDataRequest.open("GET", this._queue[0].atlasDataRequest.jsonURL, true);
					this._queue[0].atlasDataRequest.send();
					break;

				// TODO Audio
			}
		} else
		{
			if(this.preloading == true)
			{
				Debug.warn("ManagerAsset* currently in preload queue");
			}
		}
	}



	public getWithTag(tag:string)
	{
		if(this._assets[tag] != undefined)
		{
			return this._assets[tag];
		} else
		{
			Debug.warn("ManagerAsset* tag not used: " + tag);
		}
	}



	private preloadNextItem(scope):void
	{
		// remove from queue, place in assets
		var asset:Asset = this._queue[0];
		scope._queue = scope._queue.splice(1, scope._queue.length);
		scope._assets[asset.tag] = asset;

		Debug.log("ManagerAsset* preloaded: " + asset.url);

		// TODO any prep needed post load (i.e. convert image to canvas)
		if(asset.type == ManagerAsset.TYPE_IMAGE)
		{
			/*var canvas = document.createElement("canvas");
			var ctx = canvas.getContext("2d");
			ctx.drawImage(asset.content);

			asset.content = canvas;*/
		}


		// other items in the queue?
		if(scope._queue.length > 0)
		{
			switch(scope._queue[0].type)
			{
				case ManagerAsset.TYPE_IMAGE:
					scope._queue[0].content.src = scope._queue[0].url;
					break;

				case ManagerAsset.TYPE_ATLAS:
					scope._queue[0].atlasDataRequest.open("GET", this._queue[0].atlasDataRequest.jsonURL, true);
					scope._queue[0].atlasDataRequest.send();
					break;

				// TODO Audio
			}
		} else
		{
			scope.preloading = false;
			Debug.log("ManagerAsset* preload complete");

			this._returnFunction();
		}
	}
}