class ManagerSystem
{
	// members
	public static instance:ManagerSystem = null;

	private _systems_input:System[]      = null;
	private _systems_update:System[]     = null;
	private _systems_collision:System[]  = null;
	private _systems_render:System[]     = null;

	// constants for system priority
	static PRIORITY_INPUT:string         = "priority_input";
	static PRIORITY_UPDATE:string        = "priority_update";
	static PRIORITY_COLLISION:string     = "priority_collision";
	static PRIORITY_RENDER:string        = "priority_render";


	// constructor
	constructor()
	{
		// setup instance
		if(ManagerSystem.instance == null)
		{
				ManagerSystem.instance  = this;

				this._systems_input     = new System[];
				this._systems_update    = new System[];
				this._systems_collision = new System[];
				this._systems_render    = new System[];
		}

		Debug.log("ManagerSystem* init");
	}


	// adds a system
	// TODO priority needed, pre-update, update, post-update, collision, react, render
	public addWithTag(system:System, tag:string, priority:string):void
	{
		var prioritisedSystem:System[];
		switch(priority)
		{
				case ManagerSystem.PRIORITY_INPUT:
					prioritisedSystem = this._systems_input;
					break;

				case ManagerSystem.PRIORITY_UPDATE:
					prioritisedSystem = this._systems_update;
					break;

				case ManagerSystem.PRIORITY_COLLISION:
					prioritisedSystem = this._systems_collision;
					break;

				case ManagerSystem.PRIORITY_RENDER:
					prioritisedSystem = this._systems_render;
					break;
		}

		if(system[tag] == undefined)
		{
			system.tag = tag;
			prioritisedSystem[tag] = system;
			system.start();

			Debug.log("ManagerSystem* added system: " + tag);
		} else
		{
			Debug.warn("ManagerSystem* system tag already used: " + tag);
		}
	}


	public getWithTag(tag:string):System
	{
		if(this._systems_input[tag] != undefined)
		{
			return this._systems_input[tag];
		} else if(this._systems_update[tag] != undefined)
		{
			return this._systems_update[tag];
		} else if(this._systems_collision[tag] != undefined)
		{
			return this._systems_collision[tag];
		} else if(this._systems_render[tag] != undefined)
		{
			return this._systems_render[tag];
		} else
		{
			Debug.warn("ManagerSystem* tag not found: " + tag);
		}
	}


	// iterates over each System with the updated time
	public update(dt:number, t:number):void
	{
		for(var system in this._systems_input)
		{
			if(this._systems_input[system] != undefined)
			{
				this._systems_input[system].update(dt, t);
			}
		}
		for(var system in this._systems_update)
		{
			if(this._systems_update[system] != undefined)
			{
				this._systems_update[system].update(dt, t);
			}
		}
		for(var system in this._systems_collision)
		{
			if(this._systems_collision[system] != undefined)
			{
				this._systems_collision[system].update(dt, t);
			}
		}
		for(var system in this._systems_render)
		{
			if(this._systems_render[system] != undefined)
			{
				this._systems_render[system].update(dt, t);
			}
		}
	}


	// removes a system
	public removeWithTag(tag:string):void
	{
		if(this._systems_input[tag] != undefined)
		{
			this._systems_input[tag].end();
			this._systems_input[tag] = undefined;

			Debug.log("ManagerSystem* removed system: " + tag);
		} else if(this._systems_update[tag] != undefined)
		{
			this._systems_update[tag].end();
			this._systems_update[tag] = undefined;

			Debug.log("ManagerSystem* removed system: " + tag);
		} else if(this._systems_collision[tag] != undefined)
		{
			this._systems_collision[tag].end();
			this._systems_collision[tag] = undefined;

			Debug.log("ManagerSystem* removed system: " + tag);
		} else if(this._systems_render[tag] != undefined)
		{
			this._systems_render[tag].end();
			this._systems_render[tag] = undefined;

			Debug.log("ManagerSystem* removed system: " + tag);
		} else
		{
			Debug.warn("ManagerSystem* tag not found: " + tag);
		}
	}
}
