/**
 * SystemCanvasRenderer
 */
class SystemCanvasRenderer extends System
{
	// properties
	private _screenData:ScreenData 				= null;
	private _ctx								= null;;


	// constructor
	constructor(screenData:ScreenData)
	{
		super();

		this._screenData 	= screenData;
		this._ctx 			= screenData.canvas.getContext('2d');
		this.nodelist 		= ManagerSystemNode.instance.getList(NodeRenderable);

		Debug.log("SystemCanvasRenderer* init complete");
	}


	public start():void
	{
		Debug.log("SystemCanvasRenderer* start");
	}


	public update(dt:number, t:number):void
	{
		this._ctx.clearRect(0, 0, this._screenData.width, this._screenData.height);

		for(var node = this.nodelist.head; node; node = node.next)
		{
			this._ctx.save();

			// basic display object
			if(node.animation == null)
			{
				this._ctx.translate(node.position.x, node.position.y);
				this._ctx.rotate(node.position.rotation * Math.PI / 180);


				this._ctx.drawImage(
					// view
					node.display.view,

					// src x/y w/h
					0,
					0,
					node.display.view.width,
					node.display.view.height,

					// dest x/y w/h
					-node.position.origin.x,
					-node.position.origin.y,
					node.display.width,
					node.display.height
				);
			} else
			{
				if(node.animation.currentFrame != undefined)
				{
					// Atlas
					this._ctx.translate(node.position.x, node.position.y);
					this._ctx.rotate(node.position.rotation * Math.PI / 180);

					this._ctx.drawImage(
						// view
						node.display.view,

						// src x/y w/h
						node.animation.currentFrame.frameX,
						node.animation.currentFrame.frameY,
						node.animation.currentFrame.frameWidth,
						node.animation.currentFrame.frameHeight,

						// dest x/y w/h
						-node.position.origin.x,
						-node.position.origin.y,
						node.display.width,
						node.display.height
					);
				}

			}

			this._ctx.restore();
		}
	}


	public end():void
	{
		Debug.log("SystemCanvasRenderer* end");
	}
}