class Sprite extends Entity
{
	// properties
	public position:ComponentPosition		= null;
	public display:ComponentDisplay			= null;
	public animation:ComponentAnimation		= null;
	public velocity:ComponentVelocity		= null;



	/**
	 * constructor
	 *
	 * tag 		- string identifier of the Entity
	 * img 		- image name from the AssetManager
	 * atlas 	- if the image is an atlas
	 */
	constructor(tag:string, img:string, atlas:bool)
	{
		super(tag);

		// initialise components
		this.position 	= new ComponentPosition();
		this.display 	= new ComponentDisplay();
		this.velocity	= new ComponentVelocity();

		// node creation
		var renderNode:NodeRenderable 			= new NodeRenderable(this);
		renderNode.display 						= this.display;
		renderNode.position 					= this.position;

		// node creation
		var movementNode:NodeMovement 			= new NodeMovement(this);
		movementNode.velocity 					= this.velocity;
		movementNode.position 					= this.position;

		// register nodes to systems
		ManagerSystemNode.instance.addToList(renderNode, NodeRenderable);
		ManagerSystemNode.instance.addToList(movementNode, NodeMovement);

		// synthesize Entity with components
		this.componentAddWithTag(this.position, 	"position");
		this.componentAddWithTag(this.display, 		"display");
		this.componentAddWithTag(this.velocity, 	"velocity");

		// add display object
		this.display.view = ManagerAsset.instance.getWithTag(img).content;

		// if using an atlas, add extra props
		if(atlas)
		{
			this.animation 	= new ComponentAnimation();

			renderNode.animation 					= this.animation;

			var animationNode:NodeAnimation 		= new NodeAnimation(this);
			animationNode.animation 				= this.animation;
			animationNode.display 					= this.display;

			this.animation.frames 		= ManagerAsset.instance.getWithTag(img).atlasData;

			this.display.width			= this.animation.currentFrame.frameWidth;
			this.display.height			= this.animation.currentFrame.frameHeight;

			ManagerSystemNode.instance.addToList(animationNode, NodeAnimation);

			this.componentAddWithTag(this.animation,	"animation");
		}

		// add to ManagerEntity
		ManagerEntity.instance.addWithTag(this, this.tag());
	}
}