/**
 * Game
 */
class Game extends Engine
{
	// properties
	private _keyInput:KeyInput 						= null;
	private _screenData:ScreenData 					= null;


	// constructor
	constructor(canvas, width, height)
	{
		super(canvas, width, height);

		// Debug
		//Debug.debuggingEnabled = false;

		// create the screen bounds
		this._screenData 		= new ScreenData(canvas, width, height);

		// keyboard
		this._keyInput 					= new KeyInput();

		// preload assets
		this.managerAsset.addWithTag("./bin/assets/image/bg.png", 			ManagerAsset.TYPE_IMAGE, "bg");
		this.managerAsset.addWithTag("./bin/assets/image/ship.png", 	ManagerAsset.TYPE_ATLAS, "shipAtlas");
		this.managerAsset.queueLoad(() => this.postPreload());
	}



	public postPreload():void
	{
		// systems init
		this.managerSystem.addWithTag(new SystemKeyboardInput(this._keyInput, this._screenData), "main", ManagerSystem.PRIORITY_INPUT);

		// create entities
		this._screenData.setBackgroundImage("./bin/assets/image/bg.png", false);

		var ship:Sprite 		= new Sprite("ship", "shipAtlas", true);
		ship.position.origin.x 	= 9;
		ship.position.origin.y 	= 11;
		ship.position.x			= 400;
		ship.position.y			= 250;
		//ship.velocity.damping	= 0.99;

		ship.animation.addWithTag("boost", ["asteroids1.png"]);
		ship.animation.addWithTag("noboost", ["asteroids0.png"]);

		// register the ship with a node and a system that controls it
		var shipNode:ShipNode 	= new ShipNode(ship);
		shipNode.position		= ship.position;
		shipNode.display		= ship.display;
		shipNode.velocity		= ship.velocity;
		shipNode.animation		= ship.animation;

		ManagerSystemNode.instance.addToList(shipNode, ShipNode);

		// start the core
		this.setCoreRunning(true);
	}
}