/**
 * Game
 */
class Game extends Engine
{
	// properties
	private _screenData:ScreenData 					= null;


	// constructor
	constructor(canvas, width, height)
	{
		super(canvas, width, height);

		// Debug
		//Debug.debuggingEnabled = false;

		// create the screen bounds
		this._screenData 		= new ScreenData(canvas, width, height);

		// set screen bounds
		this._screenData.canvas.width 	= width;
		this._screenData.canvas.height 	= height;

		// preload assets
		this.managerAsset.addWithTag("./bin/assets/image/bg.png", ManagerAsset.TYPE_IMAGE, "bg");
		this.managerAsset.addWithTag("./bin/assets/image/invader.png", ManagerAsset.TYPE_IMAGE, "invader");
		this.managerAsset.addWithTag("./bin/assets/image/animatedInvader.png", ManagerAsset.TYPE_ATLAS, "invaderAtlas");
		this.managerAsset.queueLoad(() => this.postPreload());
	}



	public postPreload():void
	{
		// systems init
		this.managerSystem.addWithTag(new SystemOriginPoint(), "main", ManagerSystem.PRIORITY_UPDATE);

		// add bg
		this._screenData.setBackgroundImage("./bin/assets/image/bg.png", true);

		// create entities
		var invader:Sprite 					= this.createInvader("invaderEntity", false, false, 175, 257);
		var invaderCentre:Sprite 			= this.createInvader("invaderEntityCentre", false, true, 378, 257);
		var invaderCentreAtlas:Sprite 		= this.createInvader("invaderEntityCentreAtlas", true, true, 587, 258);

		// start the core
		this.setCoreRunning(true);
	}



	// convenience in entity creation for this test
	public createInvader(tag:string, animated:bool, centreOrigin:bool, x:number, y:number):Sprite
	{
		var invader:Sprite;

		if(animated)
		{
			invader = new Sprite(tag, "invaderAtlas", animated);
			invader.animation.addWithTag("idle", ["invader.png", "invader1.png"]);
			invader.animation.currentAnimationTag = "idle";
			invader.animation.frameskip = 14;
		} else
		{
			invader = new Sprite(tag, "invader", animated);
		}

		// add to the tests system
		var invaderNode:NodeLogic		= new NodeLogic(invader);
		invaderNode.position			= invader.position;
		invaderNode.display				= invader.display;

		ManagerSystemNode.instance.addToList(invaderNode, NodeLogic);

		// offsetting
		if(centreOrigin)
		{
			invader.position.origin.x 	= 22;
			invader.position.origin.y 	= 16;
		}

		invader.position.x 				= x;
		invader.position.y 				= y;

		return invader;
	}
}