/**
 * Game
 */
class Game extends Engine
{
	// properties
	private _keyInput:KeyInput 						= null;
	private _screenData:ScreenData 					= null;


	// constructor
	constructor(canvas, width, height)
	{
		super(canvas, width, height);

		// Debug
		//Debug.debuggingEnabled = false;

		// create the screen bounds
		this._screenData 		= new ScreenData(canvas, width, height);

		// keyboard
		this._keyInput 					= new KeyInput();

		// preload assets
		this.managerAsset.addWithTag("./bin/assets/image/bg.png", 			ManagerAsset.TYPE_IMAGE, "bg");
		this.managerAsset.addWithTag("./bin/assets/image/asteroid.png", 	ManagerAsset.TYPE_ATLAS, "asteroidAtlas");
		this.managerAsset.queueLoad(() => this.postPreload());
	}



	public postPreload():void
	{
		// systems init
		this.managerSystem.addWithTag(new SystemSimpleCollison(this._screenData), "main", ManagerSystem.PRIORITY_INPUT);

		// create entities
		this._screenData.setBackgroundImage("./bin/assets/image/bg.png", false);

		for(var i:number = 0; i < 10; i++)
		{
			var asteroid:Sprite 		= new Sprite("asteroid" + i, "asteroidAtlas", true);
			asteroid.position.x			= Math.random() * this._screenData.width;
			asteroid.position.y			= Math.random() * this._screenData.height;
			asteroid.position.origin.x 	= 9;
			asteroid.position.origin.y 	= 11;
			asteroid.velocity.x			= -50 + Math.random() * 100;
			asteroid.velocity.y			= -50 + Math.random() * 100;
			asteroid.velocity.damping	= 1;

			asteroid.animation.addWithTag("type1", ["asteroid1.png"]);
			asteroid.animation.addWithTag("type2", ["asteroid2.png"]);
			asteroid.animation.addWithTag("type3", ["asteroid3.png"]);
			asteroid.animation.addWithTag("type4", ["asteroid4.png"]);

			// set random frame
			var frame:number = Math.round(Math.random() * 3) + 1;
			asteroid.animation.currentAnimationTag = "type" + frame;

			// register the asteroid with a node and a system that controls it
			var asteroidNode:AsteroidNode 	= new AsteroidNode(asteroid);
			asteroidNode.position			= asteroid.position;
			asteroidNode.display			= asteroid.display;
			asteroidNode.velocity			= asteroid.velocity;
			asteroidNode.animation			= asteroid.animation;

			ManagerSystemNode.instance.addToList(asteroidNode, AsteroidNode);
		}

		// start the core
		this.setCoreRunning(true);
	}
}